@@ -7,6 +7,7 @@
 #' @importFrom stats na.omit
 .onAttach = function(libname, pkgname) {
   inst.ver = as.character(utils::packageVersion("FMAT"))
+  pkg.date = substr(utils::packageDate("FMAT"), 1, 4)
   pkgs = c("data.table", "stringr", "forcats")
   suppressMessages({
     suppressWarnings({
@@ -25,7 +26,17 @@
     {magenta Online documentation:}
     {underline https://psychbruce.github.io/FMAT}
 
-    "))
+    {magenta To use this package in publications, please cite:}
+    Bao, H.-W.-S. ({pkg.date}). "),
+    glue::glue_col("{italic FMAT: The Fill-Mask Association Test}"),
+    glue::glue_col(" (Version {inst.ver}) [Computer software]. "),
+    glue::glue_col("{underline https://CRAN.R-project.org/package=FMAT}"),
+    "\n\n",
+    glue::glue_col("Bao, H.-W.-S. (2024). The Fill-Mask Association Test (FMAT): "),
+    glue::glue_col("Measuring propositions in natural language. "),
+    glue::glue_col("{italic Journal of Personality and Social Psychology. }"),
+    glue::glue_col("Advance online publication. {underline https://doi.org/10.1037/pspa0000396}"),
+    "\n")
   }
 }
 
@@ -38,7 +49,7 @@
 PsychWordVec::cc
 
 
-#' A simple function equivalent to \code{list}.
+#' A simple function equivalent to `list`.
 #'
 #' @param ... Named objects (usually character vectors for this package).
 #'
@@ -52,28 +63,6 @@ PsychWordVec::cc
 . = function(...) list(...)
 
 
-warning.init = "
-    No valid Python or conda environment.
-
-    You may need to specify the version of Python:
-      RStudio -> Tools -> Global/Project Options
-      -> Python -> Select -> Conda Environments
-      -> Choose \".../textrpp_condaenv/python.exe\"
-
-    You may also use `PsychWordVec::text_init()`
-    to initialize the Python environment."
-
-
-text_initialized = function() {
-  error = TRUE
-  try({
-    text::textModels()
-    error = FALSE
-  }, silent=TRUE)
-  if(error) PsychWordVec::text_init()
-}
-
-
 dtime = function(t0) {
   diff = as.numeric(difftime(Sys.time(), t0, units="secs"))
   if(diff < 1) {
@@ -88,56 +77,166 @@ dtime = function(t0) {
 }
 
 
+gpu_to_device = function(gpu) {
+  cuda = reticulate::import("torch")$cuda$is_available()
+  if(missing(gpu))
+    gpu = cuda
+  if(is.logical(gpu))
+    device = ifelse(gpu, 0L, -1L)
+  if(is.numeric(gpu))
+    device = as.integer(device)
+  if(is.character(gpu))
+    device = gpu
+  if(!device %in% c(-1L, "cpu") & !cuda)
+    stop("
+      NVIDIA GPU CUDA is not enabled!
+      For guidance, see https://psychbruce.github.io/FMAT/",
+      call.=FALSE)
+  return(device)
+}
+
+
+transformers_init = function() {
+  reticulate::py_capture_output({
+    os = reticulate::import("os")
+    os$environ["TF_ENABLE_ONEDNN_OPTS"] = "0"
+    os$environ["HF_HUB_DISABLE_SYMLINKS_WARNING"] = "1"
+
+    torch = reticulate::import("torch")
+    torch.ver = torch$`__version__`
+    torch.cuda = torch$cuda$is_available()
+    if(torch.cuda) {
+      cuda.ver = torch$cuda_version
+      gpu.info = paste("GPU (Device):", paste(torch$cuda$get_device_name(), collapse=", "))
+    } else {
+      cuda.ver = "NULL"
+      gpu.info = paste("(To use GPU, install PyTorch with CUDA support,",
+                       "see https://pytorch.org/get-started)")
+    }
+
+    transformers = reticulate::import("transformers")
+    tf.ver = transformers$`__version__`
+  })
+  cli::cli_alert_info(cli::col_blue("Device Info:
+
+  Python Environment:
+  Package       Version
+  transformers  {tf.ver}
+  torch         {torch.ver}
+
+  NVIDIA GPU CUDA Support:
+  CUDA Enabled: {torch.cuda}
+  CUDA Version: {cuda.ver}
+  {gpu.info}
+  "))
+  return(transformers)
+}
+
+
+find_cached_models = function(cache.folder) {
+  models.name = list.files(cache.folder, "^models--")
+  if(length(models.name) > 0) {
+    models.size = sapply(paste0(cache.folder, "/", models.name), function(folder) {
+      models.file = list.files(folder, pattern="(model.safetensors$|pytorch_model.bin$|tf_model.h5$)", recursive=TRUE, full.names=TRUE)
+      paste(paste0(sprintf("%.0f", file.size(models.file) / 1024^2), " MB"), collapse=" / ")
+    })
+    models.name = str_replace_all(str_remove(models.name, "^models--"), "--", "/")
+    models.info = data.frame(Size=models.size, row.names=models.name)
+  } else {
+    models.info = NULL
+  }
+  return(models.info)
+}
+
+
+#### BERT ####
+
+
+#' Download and save BERT models to local cache folder.
+#'
+#' Download and save BERT models to local cache folder "%USERPROFILE%/.cache/huggingface".
+#'
+#' @param models Model names at
+#' [HuggingFace](https://huggingface.co/models?pipeline_tag=fill-mask&library=transformers).
+#'
+#' @return
+#' No return value.
+#'
+#' @seealso
+#' [`FMAT_load`]
+#'
+#' @examples
+#' \dontrun{
+#' model.names = c("bert-base-uncased", "bert-base-cased")
+#' BERT_download(model.names)
+#'
+#' BERT_download()  # check downloaded models
+#' }
+#'
+#' @export
+BERT_download = function(models=NULL) {
+  transformers = transformers_init()
+  if(!is.null(models)) {
+    lapply(models, function(model) {
+      cli::cli_h1("Downloading model {.val {model}}")
+      cli::cli_alert("(1) Downloading configuration...")
+      transformers$AutoConfig$from_pretrained(model)
+      cli::cli_alert("(2) Downloading tokenizer...")
+      transformers$AutoTokenizer$from_pretrained(model)
+      cli::cli_alert("(3) Downloading model...")
+      transformers$AutoModel$from_pretrained(model)
+      cli::cli_alert_success("Successfully downloaded model {.val {model}}")
+      gc()
+    })
+  }
+  cache.folder = str_replace_all(transformers$TRANSFORMERS_CACHE, "\\\\", "/")
+  cache.sizegb = sum(file.size(list.files(cache.folder, recursive=TRUE, full.names=TRUE))) / 1024^3
+  local.models = find_cached_models(cache.folder)
+  cli::cli_h2("Downloaded models:")
+  print(local.models)
+  cat("\n")
+  cli::cli_alert_success("Downloaded models saved at {.path {cache.folder}} ({sprintf('%.2f', cache.sizegb)} GB)")
+}
+
+
 #### FMAT ####
 
 
-#' Initialize running environment and (down)load language models.
+#' (Down)Load BERT models (useless for GPU).
 #'
-#' @param models Language model names (usually the BERT-based models) at
-#' \href{https://huggingface.co/models}{HuggingFace}.
+#' Load BERT models from local cache folder "%USERPROFILE%/.cache/huggingface".
+#' Models that have not been downloaded can also
+#' be automatically downloaded (but *silently*).
+#' For [GPU Acceleration](https://psychbruce.github.io/FMAT/#guidance-for-gpu-acceleration),
+#' please directly use [`FMAT_run`] instead.
 #'
-#' For a full list of available models, see
-#' \url{https://huggingface.co/models?pipeline_tag=fill-mask&library=transformers}
+#' @inheritParams BERT_download
 #'
 #' @return
 #' A named list of fill-mask pipelines obtained from the models.
-#' The returned object \emph{cannot} be saved as any RData.
-#' You will need to \emph{rerun} this function if you restart the R session.
-#'
-#' All downloaded models are saved
-#' at your local folder "C:/Users/[YourUserName]/.cache/".
+#' The returned object *cannot* be saved as any RData.
+#' You will need to *rerun* this function if you *restart* the R session.
 #'
 #' @seealso
-#' \code{\link[PsychWordVec:text_init]{PsychWordVec::text_init}}
+#' [`BERT_download`]
 #'
-#' \code{\link{FMAT_query}}
+#' [`FMAT_query`]
 #'
-#' \code{\link{FMAT_query_bind}}
+#' [`FMAT_query_bind`]
 #'
-#' \code{\link{FMAT_run}}
+#' [`FMAT_run`]
 #'
 #' @examples
-#' \donttest{models = FMAT_load(c("bert-base-uncased", "bert-base-cased"))
+#' \dontrun{
+#' model.names = c("bert-base-uncased", "bert-base-cased")
+#' models = FMAT_load(model.names)  # load models from cache
 #' }
+#'
 #' @export
 FMAT_load = function(models) {
-  cli::cli_text("Initializing environment...")
-  try({
-    error = TRUE
-    text_initialized()
-    error = FALSE
-  }, silent=TRUE)
-  if(error) {
-    warning(warning.init, call.=FALSE)
-    return(NULL)
-  }
-  old.models = text::textModels()$Downloaded_models
-  new.models = setdiff(models, old.models)
-  if(length(new.models) > 0)
-    PsychWordVec::text_model_download(new.models)
-
-  cli::cli_text("Loading models...")
-  transformers = reticulate::import("transformers")
+  transformers = transformers_init()
+  cache.folder = str_replace_all(transformers$TRANSFORMERS_CACHE, "\\\\", "/")
+  cli::cli_text("Loading models from {.path {cache.folder}} ...")
   fms = lapply(models, function(model) {
     t0 = Sys.time()
     reticulate::py_capture_output({
@@ -231,37 +330,38 @@ append_X = function(dq, X, var="TARGET") {
 #' Prepare a data.table of queries and variables for the FMAT.
 #'
 #' @param query Query text (should be a character string/vector
-#' with at least one \code{[MASK]} token).
+#' with at least one `[MASK]` token).
 #' Multiple queries share the same set of
-#' \code{MASK}, \code{TARGET}, and \code{ATTRIB}.
+#' `MASK`, `TARGET`, and `ATTRIB`.
 #' For multiple queries with different
-#' \code{MASK}, \code{TARGET}, and/or \code{ATTRIB},
-#' please use \code{\link{FMAT_query_bind}} to combine them.
-#' @param MASK A named list of \code{[MASK]} target words.
+#' `MASK`, `TARGET`, and/or `ATTRIB`,
+#' please use [`FMAT_query_bind`] to combine them.
+#' @param MASK A named list of `[MASK]` target words.
 #' Must be single words in the vocabulary of a certain masked language model.
-#' For model vocabulary, see, e.g.,
-#' \url{https://huggingface.co/bert-base-uncased/raw/main/vocab.txt}
 #'
-#' Note that infrequent words may be not included in a model's vocabulary,
+#' For model vocabulary, see, e.g.,
+#' <https://huggingface.co/bert-base-uncased/raw/main/vocab.txt>
+#'
+#' Infrequent words may be not included in a model's vocabulary,
 #' and in this case you may insert the words into the context by
-#' specifying either \code{TARGET} or \code{ATTRIB}.
+#' specifying either `TARGET` or `ATTRIB`.
 #' @param TARGET,ATTRIB A named list of Target/Attribute words or phrases.
-#' If specified, then \code{query} must contain
-#' \code{{TARGET}} and/or \code{{ATTRIB}} (in all uppercase and in braces)
+#' If specified, then `query` must contain
+#' `{TARGET}` and/or `{ATTRIB}` (in all uppercase and in braces)
 #' to be replaced by the words/phrases.
-#' @param unmask.id If there are multiple \code{[MASK]} in \code{query},
-#' this argument will be used to determine which one is to be unmasked.
-#' Defaults to the 1st \code{[MASK]}.
+#' @param unmask.id If multiple `[MASK]` are in `query`,
+#' it determines which one should be unmasked.
+#' Defaults to the 1st `[MASK]`.
 #'
 #' @return
 #' A data.table of queries and variables.
 #'
 #' @seealso
-#' \code{\link{FMAT_load}}
+#' [`FMAT_load`]
 #'
-#' \code{\link{FMAT_query_bind}}
+#' [`FMAT_query_bind`]
 #'
-#' \code{\link{FMAT_run}}
+#' [`FMAT_run`]
 #'
 #' @examples
 #' FMAT_query("[MASK] is a nurse.", MASK = .(Male="He", Female="She"))
@@ -345,17 +445,17 @@ FMAT_query = function(
 
 #' Combine multiple query data.tables and renumber query ids.
 #'
-#' @param ... Query data.tables returned from \code{\link{FMAT_query}}.
+#' @param ... Query data.tables returned from [`FMAT_query`].
 #'
 #' @return
 #' A data.table of queries and variables.
 #'
 #' @seealso
-#' \code{\link{FMAT_load}}
+#' [`FMAT_load`]
 #'
-#' \code{\link{FMAT_query}}
+#' [`FMAT_query`]
 #'
-#' \code{\link{FMAT_run}}
+#' [`FMAT_run`]
 #'
 #' @examples
 #' FMAT_query_bind(
@@ -402,79 +502,88 @@ FMAT_query_bind = function(...) {
 # stopCluster(cl)
 
 
-#' Run the fill-mask pipeline on multiple models.
+#' Run the fill-mask pipeline on multiple models (CPU / GPU).
+#'
+#' Run the fill-mask pipeline on multiple models with CPU or GPU
+#' (faster but requiring an NVIDIA GPU device).
 #'
 #' @details
-#' The function will also automatically adjust for
+#' The function automatically adjusts for
 #' the compatibility of tokens used in certain models:
 #' (1) for uncased models (e.g., ALBERT), it turns tokens to lowercase;
-#' (2) for models that use \code{<mask>} rather than \code{[MASK]},
+#' (2) for models that use `<mask>` rather than `[MASK]`,
 #' it automatically uses the corrected mask token;
 #' (3) for models that require a prefix to estimate whole words than subwords
 #' (e.g., ALBERT, RoBERTa), it adds a certain prefix (usually a white space;
 #' \\u2581 for ALBERT and XLM-RoBERTa, \\u0120 for RoBERTa and DistilRoBERTa).
 #'
-#' Note that these changes only affect the \code{token} variable
-#' in the returned data, but will not affect the \code{M_word} variable.z
-#' Thus, users may analyze their data based on the unchanged \code{M_word}
-#' rather than the \code{token}.
+#' Note that these changes only affect the `token` variable
+#' in the returned data, but will not affect the `M_word` variable.
+#' Thus, users may analyze data based on the unchanged `M_word`
+#' rather than the `token`.
 #'
-#' @param models Language model(s):
-#' \itemize{
-#'   \item{Model names (usually the BERT-based models) at
-#'    \href{https://huggingface.co/models}{HuggingFace}.}
-#'   \item{A list of mask filling pipelines loaded by \code{\link{FMAT_load}}.
+#' Note also that there may be extremely trivial differences
+#' (after 5~6 significant digits) in the
+#' raw probability estimates between using CPU and GPU,
+#' but these differences would have little impact on main results.
 #'
-#'    * You will need to \strong{rerun} \code{\link{FMAT_load}}
-#'    if you \strong{restart} the R session.}
-#' }
-#' @param data A data.table returned from
-#' \code{\link{FMAT_query}} or \code{\link{FMAT_query_bind}}.
-#' @param file File name of \code{.RData} to save the returned data.
-#' @param progress Show a progress bar:
-#' \code{"none"} (\code{FALSE}), \code{"text"} (\code{TRUE}), \code{"time"}.
-#' @param parallel Parallel processing (NOT suggested).
-#' Defaults to \code{FALSE}.
-#' If \code{TRUE}, then \code{models} must be model names
-#' rather than from \code{\link{FMAT_load}}.
+#' @param models Options:
+#' - A character vector of model names at
+#'   [HuggingFace](https://huggingface.co/models?pipeline_tag=fill-mask&library=transformers).
+#'   - Can be used for both CPU and GPU.
+#' - A returned object from [`FMAT_load`].
+#'   - Can ONLY be used for CPU.
+#'   - If you *restart* the R session,
+#'     you will need to *rerun* [`FMAT_load`].
+#' @param data A data.table returned from [`FMAT_query`] or [`FMAT_query_bind`].
+#' @param gpu Use GPU (3x faster than CPU) to run the fill-mask pipeline?
+#' Defaults to missing value that will *automatically* use available GPU
+#' (if not available, then use CPU).
+#' An NVIDIA GPU device (e.g., GeForce RTX Series) is required to use GPU.
+#' See [Guidance for GPU Acceleration](https://psychbruce.github.io/FMAT/#guidance-for-gpu-acceleration).
 #'
-#' * For small-scale \code{data},
-#' parallel processing would instead be \emph{slower}
-#' because it takes time to create a parallel cluster.
-#' @param ncores Number of CPU cores to be used in parallel processing.
-#' @param warning Warning of out-of-vocabulary word(s). Defaults to \code{TRUE}.
+#' Options passing to the `device` parameter in Python:
+#' - `FALSE`: CPU (`device = -1`).
+#' - `TRUE`: GPU (`device = 0`).
+#' - Any other value: passing to
+#'   [transformers.pipeline(device=...)](https://huggingface.co/docs/transformers/main_classes/pipelines#transformers.pipeline.device)
+#'   which defines the device (e.g.,
+#'   `"cpu"`, `"cuda:0"`, or a GPU device id like `1`)
+#'   on which the pipeline will be allocated.
+#' @param file File name of `.RData` to save the returned data.
+#' @param progress Show a progress bar? Defaults to `TRUE`.
+#' @param warning Alert warning of out-of-vocabulary word(s)? Defaults to `TRUE`.
 #'
 #' @return
-#' A data.table (of new class \code{fmat}) appending \code{data}
-#' with these new variables:
-#' \itemize{
-#'   \item{\code{model}: model name.}
-#'   \item{\code{output}: complete sentence output with unmasked token.}
-#'   \item{\code{token}: actual token to be filled in the blank mask
+#' A data.table (of new class `fmat`) appending `data` with these new variables:
+#' - `model`: model name.
+#' - `output`: complete sentence output with unmasked token.
+#' - `token`: actual token to be filled in the blank mask
 #'   (a note "out-of-vocabulary" will be added
-#'   if the original word is not found in the model vocabulary).}
-#'   \item{\code{prob}: (raw) conditional probability of the unmasked token
+#'   if the original word is not found in the model vocabulary).
+#' - `prob`: (raw) conditional probability of the unmasked token
 #'   given the provided context, estimated by the masked language model.
-#'
-#'   * It is NOT SUGGESTED to directly interpret the raw probabilities
-#'   because the \emph{contrast} between a pair of probabilities
-#'   is more interpretable. See \code{\link{summary.fmat}}.}
-#' }
+#'   - It is NOT SUGGESTED to directly interpret the raw probabilities
+#'   because the *contrast* between a pair of probabilities
+#'   is more interpretable. See [`summary.fmat`].
 #'
 #' @seealso
-#' \code{\link{FMAT_load}}
+#' [`BERT_download`]
 #'
-#' \code{\link{FMAT_query}}
+#' [`FMAT_load`]
 #'
-#' \code{\link{FMAT_query_bind}}
+#' [`FMAT_query`]
 #'
-#' \code{\link{summary.fmat}}
+#' [`FMAT_query_bind`]
+#'
+#' [`summary.fmat`]
 #'
 #' @examples
-#' # Running the example requires the models downloaded
-#' # You will need to rerun `FMAT_load` if you restart the R session
+#' ## Running the examples requires the models downloaded
 #'
-#' \donttest{models = FMAT_load(c("bert-base-uncased", "bert-base-cased"))
+#' \dontrun{
+#' models = FMAT_load(c("bert-base-uncased", "bert-base-cased"))
+#' # for GPU acceleration, please use `FMAT_run()` directly
 #'
 #' query1 = FMAT_query(
 #'   c("[MASK] is {TARGET}.", "[MASK] works as {TARGET}."),
@@ -507,44 +616,47 @@ FMAT_query_bind = function(...) {
 #' summary(data3, attrib.pair=FALSE)
 #' summary(data3)
 #' }
+#'
 #' @export
 FMAT_run = function(
     models,
     data,
+    gpu,
     file = NULL,
-    progress = c(FALSE, TRUE, "none", "text", "time"),
-    parallel = FALSE,
-    ncores = 4,
+    progress = TRUE,
     warning = TRUE
 ) {
-  if(is.null(models)) {
-    warning(warning.init, call.=FALSE)
-    return(NULL)
+  t0 = Sys.time()
+  type = attr(data, "type")
+  device = gpu_to_device(gpu)
+  progress = ifelse(progress, "text", "none")
+
+  if(inherits(models, "fill.mask")) {
+    if(!device %in% c(-1L, "cpu"))
+      stop("
+      To use GPU, please specify `models` as model names,
+      rather than the returned object from `FMAT_load()`.", call.=FALSE)
+  } else {
+    transformers = transformers_init()
+    cache.folder = str_replace_all(transformers$TRANSFORMERS_CACHE, "\\\\", "/")
+    cli::cli_text("Loading models from {.path {cache.folder}} ...")
+    cat("\n")
   }
 
-  t0 = Sys.time()
-  progress = match.arg(progress)
-  if(progress=="FALSE") progress = "none"
-  if(progress=="TRUE") progress = "text"
-  type = attr(data, "type")
-
-  text_initialized()
-  cli::cli_alert_success("Environment initialized ({dtime(t0)})")
-
   onerun = function(model, data=data) {
+    ## ---- One Run Begin ---- ##
     if(is.character(model)) {
-      t1 = Sys.time()
-      transformers = reticulate::import("transformers")
       reticulate::py_capture_output({
-        fill_mask = transformers$pipeline(task="fill-mask", model=model)
+        fill_mask = transformers$pipeline("fill-mask", model=model, device=device)
       })
-      cli::cli_h1("{model} (model loaded: {dtime(t1)})")
-    }
-    if(is.list(model)) {
+    } else {
       fill_mask = model$fill.mask
       model = model$model.name
-      cli::cli_h1("{model}")
     }
+    if(device %in% c(-1L, "cpu"))
+      cli::cli_h1("{.val {model}} (CPU)")
+    else
+      cli::cli_h1("{.val {model}} (GPU Accelerated)")
 
     uncased = str_detect(model, "uncased|albert")
     prefix.u2581 = str_detect(model, "xlm-roberta|albert")
@@ -552,6 +664,7 @@ FMAT_run = function(
     mask.lower = str_detect(model, "roberta|bertweet")
 
     unmask = function(d) {
+      ## unmask function begin ##
       if("TARGET" %in% names(d))
         TARGET = as.character(d$T_word)
       if("ATTRIB" %in% names(d))
@@ -565,52 +678,52 @@ FMAT_run = function(
       if(prefix.u2581) mask = paste0("\u2581", mask)
       if(prefix.u0120 & !mask.begin) mask = paste0("\u0120", mask)
       if(mask.lower) query = str_replace_all(query, "\\[MASK\\]", "<mask>")
-      oov = reticulate::py_capture_output({
+      out = reticulate::py_capture_output({
         res = fill_mask(query, targets=mask, top_k=1L)[[uid]]
       })
+      # UserWarning: You seem to be using the pipelines sequentially on GPU.
+      # In order to maximize efficiency please use a dataset.
       return(data.table(
         output = res$sequence,
         token = ifelse(
-          oov=="",  # no extra output from python
+          out=="" | str_detect(out, "UserWarning|GPU"),  # no extra output from python
           res$token_str,
-          paste(res$token_str, "(out-of-vocabulary)")),
+          paste(res$token_str,
+                ifelse(str_detect(out, "vocabulary"),
+                       "(out-of-vocabulary)", out))),
         prob = res$score
       ))
+      ## unmask function end ##
     }
 
-    t2 = Sys.time()
+    t1 = Sys.time()
     suppressWarnings({
-      data = plyr::adply(
-        data, 1, unmask,
-        .progress = if(parallel) "none" else progress
-      )
+      data = plyr::adply(data, 1, unmask, .progress=progress)
     })
-    cat(paste0("  (", dtime(t2), ")\n"))
+    speed = sprintf("%.0f", nrow(data) / as.numeric(difftime(Sys.time(), t1, units="mins")))
+    cat(paste0("  (", dtime(t1), ") [", speed, " queries/min]\n"))
+
+    rm(fill_mask)
+    gc()
 
     return(cbind(data.table(model=as.factor(model)), data))
+    ## ---- One Run End ---- ##
   }
 
-  cli::cli_alert_info(" Task: {length(models)} models * {nrow(data)} queries")
-
-  if(parallel) {
-    cl = parallel::makeCluster(ncores)
-    models = names(models)
-    data = rbindlist(parallel::parLapply(cl, models, onerun, data=data))
-    parallel::stopCluster(cl)
-  } else {
-    data = rbindlist(lapply(models, onerun, data=data))
-    cat("\n")
-  }
+  cli::cli_alert_info("Task: {length(models)} models * {nrow(data)} queries")
+  t0.task = Sys.time()
+  data = rbindlist(lapply(models, onerun, data=data))
+  speed = sprintf("%.0f", nrow(data) / as.numeric(difftime(Sys.time(), t0.task, units="mins")))
+  cat("\n")
   attr(data, "type") = type
   class(data) = c("fmat", class(data))
-
   gc()
-  cli::cli_alert_success("Task completed (total time cost = {dtime(t0)})")
+  cli::cli_alert_success("Task completed ({dtime(t0)}) [{speed} queries/min]")
 
   if(warning) warning_oov(data)
 
   if(!is.null(file)) {
-    if(!str_detect(file, "\\.[Rr][Dd]a|\\.[Rr][Dd]ata"))
+    if(!str_detect(file, "\\.[Rr][Dd]a(ta)?$"))
       file = paste0(file, ".RData")
     save(data, file=file)
     cli::cli_alert_success("Data saved to {.val {file}}")
@@ -635,35 +748,34 @@ warning_oov = function(data) {
 }
 
 
-#' [S3 method] Summarize the results for the FMAT.
+#' \[S3 method\] Summarize the results for the FMAT.
 #'
 #' @description
-#' Summarize the results of \emph{Log Probability Ratio} (LPR),
-#' which indicates the \emph{relative} (vs. \emph{absolute})
+#' Summarize the results of *Log Probability Ratio* (LPR),
+#' which indicates the *relative* (vs. *absolute*)
 #' association between concepts.
 #'
 #' The LPR of just one contrast (e.g., only between a pair of attributes)
-#' may \emph{not} be sufficient for a proper interpretation of the results,
+#' may *not* be sufficient for a proper interpretation of the results,
 #' and may further require a second contrast (e.g., between a pair of targets).
 #'
 #' Users are suggested to use linear mixed models
-#' (with the R packages \code{nlme} or \code{lme4}/\code{lmerTest})
+#' (with the R packages `nlme` or `lme4`/`lmerTest`)
 #' to perform the formal analyses and hypothesis tests based on the LPR.
 #'
 #' @inheritParams FMAT_run
-#' @param object A data.table (of new class \code{fmat})
-#' returned from \code{\link{FMAT_run}}.
-## @param digits Number of decimal places of output. Defaults to \code{3}.
+#' @param object A data.table (of new class `fmat`)
+#' returned from [`FMAT_run`].
 #' @param mask.pair,target.pair,attrib.pair Pairwise contrast of
-#' \code{[MASK]}, \code{TARGET}, \code{ATTRIB}?
-#' Defaults to \code{TRUE}.
+#' `[MASK]`, `TARGET`, `ATTRIB`?
+#' Defaults to `TRUE`.
 #' @param ... Other arguments (currently not used).
 #'
 #' @return
 #' A data.table of the summarized results with Log Probability Ratio (LPR).
 #'
 #' @seealso
-#' \code{\link{FMAT_run}}
+#' [`FMAT_run`]
 #'
 #' @examples
 #' # see examples in `FMAT_run`
@@ -676,11 +788,6 @@ summary.fmat = function(
     attrib.pair=TRUE,
     warning=TRUE,
     ...) {
-  if(is.null(object)) {
-    warning(warning.init, call.=FALSE)
-    return(NULL)
-  }
-
   if(warning) warning_oov(object)
   type = attr(object, "type")
   M_word = T_word = A_word = MASK = TARGET = ATTRIB = prob = LPR = NULL
@@ -755,11 +862,11 @@ summary.fmat = function(
 
 #' Reliability analysis (Cronbach's \eqn{\alpha}) of LPR.
 #'
-#' @param fmat A data.table returned from \code{\link{summary.fmat}}.
-#' @param item Reliability of multiple \code{"query"} (default),
-#' \code{"T_word"}, or \code{"A_word"}.
+#' @param fmat A data.table returned from [`summary.fmat`].
+#' @param item Reliability of multiple `"query"` (default),
+#' `"T_word"`, or `"A_word"`.
 #' @param by Variable(s) to split data by.
-#' Options can be \code{"model"}, \code{"TARGET"}, \code{"ATTRIB"},
+#' Options can be `"model"`, `"TARGET"`, `"ATTRIB"`,
 #' or any combination of them.
 #'
 #' @return A data.table of Cronbach's \eqn{\alpha}.
@@ -794,3 +901,65 @@ LPR_reliability = function(
   return(as.data.table(alphas))
 }
 
+
+#### Deprecated ####
+
+
+## Install Python modules and initialize local environment.
+##
+## Install required Python modules and
+## initialize a local "conda" environment.
+## Run this function only once after you have installed the package.
+##
+## @examples
+## \dontrun{
+## FMAT_init()  # run it only once
+##
+## # Then please specify the version of Python:
+## # RStudio -> Tools -> Global/Project Options
+## # -> Python -> Select -> Conda Environments
+## # -> Choose ".../textrpp_condaenv/python.exe"
+## }
+##
+## @export
+# FMAT_init = function() {
+#   suppressMessages({
+#     suppressWarnings({
+#       text::textrpp_install(prompt=FALSE)
+#     })
+#   })
+#   cat("\n")
+#   cli::cli_alert_success("{.pkg Successfully installed Python modules in conda environment.}")
+#
+#   try({
+#     error = TRUE
+#     suppressMessages({
+#       suppressWarnings({
+#         text::textrpp_initialize(save_profile=TRUE, prompt=FALSE)
+#       })
+#     })
+#     error = FALSE
+#   }, silent=TRUE)
+#   if(error)
+#     stop("
+#
+#       Please specify the version of Python:
+#         RStudio -> Tools -> Global/Project Options
+#         -> Python -> Select -> Conda Environments
+#         -> Choose \".../textrpp_condaenv/python.exe\"",
+#        call.=FALSE)
+#   cat("\n")
+#   cli::cli_alert_success("{.pkg Initialized the Python modules.}")
+# }
+
+
+# if(parallel) {
+#   cl = parallel::makeCluster(ncores)
+#   models = names(models)
+#   data = rbindlist(parallel::parLapply(cl, models, onerun, data=data))
+#   parallel::stopCluster(cl)
+# } else {
+#   data = rbindlist(lapply(models, onerun, data=data))
+#   cat("\n")
+# }
+
